/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.dynamic.data.mapping.io.internal;

import com.liferay.dynamic.data.mapping.io.DDMFormJSONSerializer;
import com.liferay.dynamic.data.mapping.io.DDMFormLayoutJSONSerializer;
import com.liferay.dynamic.data.mapping.model.DDMForm;
import com.liferay.dynamic.data.mapping.model.DDMFormLayout;
import com.liferay.dynamic.data.mapping.util.DDMFormFactory;
import com.liferay.dynamic.data.mapping.util.DDMFormLayoutFactory;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.json.JSONFactory;
import com.liferay.portal.kernel.json.JSONObject;

/**
 * @author Marcellus Tavares
 */
public class DDMFormFieldTypeSettingsSerializerHelper {

	public DDMFormFieldTypeSettingsSerializerHelper(
		Class<?> ddmFormFieldTypeSettings,
		DDMFormJSONSerializer ddmFormJSONSerializer,
		DDMFormLayoutJSONSerializer ddmFormLayoutJSONSerializer,
		JSONFactory jsonFactory) {

		_ddmFormFieldTypeSettings = ddmFormFieldTypeSettings;
		_ddmFormJSONSerializer = ddmFormJSONSerializer;
		_ddmFormLayoutJSONSerializer = ddmFormLayoutJSONSerializer;
		_jsonFactory = jsonFactory;
	}

	public JSONObject getSettingsJSONObject() throws PortalException {
		DDMForm ddmForm = DDMFormFactory.create(_ddmFormFieldTypeSettings);

		String serializedDDMForm = _ddmFormJSONSerializer.serialize(ddmForm);

		return _jsonFactory.createJSONObject(serializedDDMForm);
	}

	public JSONObject getSettingsLayoutJSONObject() throws PortalException {
		DDMFormLayout ddmFormLayout = DDMFormLayoutFactory.create(
			_ddmFormFieldTypeSettings);

		String serializedDDMFormLayout = _ddmFormLayoutJSONSerializer.serialize(
			ddmFormLayout);

		return _jsonFactory.createJSONObject(serializedDDMFormLayout);
	}

	private final Class<?> _ddmFormFieldTypeSettings;
	private final DDMFormJSONSerializer _ddmFormJSONSerializer;
	private final DDMFormLayoutJSONSerializer _ddmFormLayoutJSONSerializer;
	private final JSONFactory _jsonFactory;

}